% Gaussian Beam Propagation - Interface
%
% This program propagates a gaussian beam through an interface of two
% different indices of refraction.  It can be used in combination with the
% other 'GBeam' propagating programs to find the q parameter along a
% distance with several optical elements.  The position of the interface is
% plotted and the two indices of refraction are labeled.
%
% [qf]=GBeam_Inter(qi,lambda,n1,n2,R,phi,PlotHandle,z,TanOrSag)
%
% REQUIRED INPUTs
%
% qi = Initial q parameter before interface
%
% lambda = Wavelength of beam in vacuum
% 
% n1 = Index of refraction beam starts out in
%
% n2 = Index of refraction beam ends up in
%
% R = Radius of curvature of interface.  For a beam traveling from left to
%     right, R>0 is concave to the left.
%
% phi = Angle that the interface is tilted in plane of incidence
%
%
% z = Position of the interface.  Used to plot the position of the interface when
%     propagating a beam through several elements.
%
% TanOrSag = This input needs to either be 'Tan' or 'Sag' to indicate
%            whether the tangential or sagittal plane is being calculated.
%
% VARIABLE INPUTS
%
% PlotElement = This needs to be followed by 'yes' or 'no'.  It determines
%                whether or not to plot the element on the plot specified
%                by 'PlotHandle'.
%
% PlotHandle = This is used to identify the plot that is being used so that
%              several elements can be plotted together.
%
% OUTPUTS
%
% qf = The final q parameter after the interface
%
%


function [qf]=GBeam_Inter(qi,lambda,n1,n2,R,phi,z,TanOrSag,varargin)

% This are the possible options available with a default value
Options=struct('PlotElement','on',...
               'PlotHandle',1);

% This stores just the option names to use later
OptionNames=fieldnames(Options);

% This is how many variable arguments you put in
nargs=length(varargin);

% This guarantees that for a given parameter, there is a parameter value.
if round(nargs/2)~=nargs/2
    error('Error: You must enter optional arguments as parameter name/parameter value pairs.')
end

% This goes through the variable arguments and assigns their new values
% according to the input.
for Pair=reshape(varargin,2,[])
    InputName=(Pair{1}); 
    if any(strncmp(InputName,(OptionNames),100)) % This sees if any of the inputs match the possible options
        
        if strcmp(InputName,'PlotElement')
            if ischar(Pair{2})~=1
                error('PlotElement option needs values of either ''On'' or ''Off''')
            end
        end
            
        Options.(InputName)=Pair{2}; % This assigns the value 
    else
        error([InputName,' is not a valid option'])
    end
end

if strcmp(TanOrSag,'Tan')
    M = [       sqrt((n2/n1)^2-sin(phi)^2)/(n2/n1*cos(phi)) ,                                            0                  ;
        (cos(phi)-sqrt((n2/n1)^2-sin(phi)^2))/(R*cos(phi)*sqrt((n2/n1)^2-sin(phi)^2)), cos(phi)/sqrt((n2/n1)^2-sin(phi)^2) ];
elseif strcmp(TanOrSag,'Sag')
    M = [                     1,                           0   ;
        (cos(phi)-sqrt((n2/n1)^2-sin(phi)^2))/(R*n2/n1), n1/n2];
end

% Propagation Equation
qf=(M(1,1)*qi+M(1,2))...
  /(M(2,1)*qi+M(2,2));

% Find a waist so the lens can be plotted in comparison to something
w=sqrt(-lambda./(pi*n2*imag(1./qf)));

% Line=[0,0;1.1*w,-1.1*w];
% RotLine(:,1)=[cos(phi),-sin(phi);sin(phi),cos(phi)]*[Line(:,1)];
% RotLine(:,2)=[cos(phi),-sin(phi);sin(phi),cos(phi)]*[Line(:,2)];
% RotLine(1,:)=RotLine(1,:)+z;

if strcmp(Options.PlotElement,'On')
    figure(Options.PlotHandle)
    line([z,z],[1.1*w,-1.1*w],'Color','b','LineStyle','--');
    % line(RotLine(1,:),RotLine(2,:),'Color','b');
    hold all
    text(z,1.1*w,['$n_1$=',num2str(n1),'$\;\;$'],'HorizontalAlignment','right','VerticalAlignment','bottom','Interpreter','Latex')
    text(z,1.1*w,['$\;\;n_2$=',num2str(n2)],'HorizontalAlignment','left','VerticalAlignment','bottom','Interpreter','Latex')
    text(z,-1.1*w,['$\;\;R$=',num2str(R)],'HorizontalAlignment','center','VerticalAlignment','top','Interpreter','Latex')
end

end















