% Gaussian Beam Propagation - Translation
%
% This function propagates a gaussian beam through a medium with some index
% of refraction.  It is able to find q along the z direction for any number
% of points.  It can be used with the other propagation programs to plot
% the beam profile along any number of optical elements.
%
% [qf,w,R,zf]=GBeam_L(qi,lambda,n,L,N,PlotHandle,zi)
%
% REQUIRED INPUTS
%
% qi = Initial q parameter of beam
%
% lambda = wavelength of light in vacuum
%
% n = Index of refraction of medium
%
% L = Distance beam is propagated through
%
% N = Number of points that are plotted.  For a given N, the plotted points
%     are evenly spaced along the z direction.
% 
%
% zi = Initial z value.  This needed to properly calculate the q parameter
%      and plot the profile along the correct length on the composite plot
%      when the
%
% VARIABLE INPUTS
%
% PlotElement = This needs to be followed by 'yes' or 'no'.  It determines
%                whether or not to plot the element on the plot specified
%                by 'PlotHandle'.
%
% PlotHandle = This is used to identify the plot that is being used so that
%              several elements can be plotted together.
%
% OUTPUTS
%
% qf = This is an array of q parameters along the z direction.  The length
%      and resolution of this array is determined by L and N.
%
% w = This is an array of the waists along the z direction.  This along
%     with the radius of curvature, R, are calculated from the q parameter.
%     This is what is plotted to show the beam profile.
%
% R = This is an array of the radii of curvature along the z direction.
%
% zf = The final z value.  It is used for reference when plotting the beam
%      profile through several elements.
%

function [qf,w,R,zf]=GBeam_L(qi,lambda,n,L,N,zi,varargin)

% This are the possible options available with a default value
Options=struct('PlotElement','on',...
               'PlotHandle',1,...
               'PlotColor','red');

% This stores just the option names to use later
OptionNames=fieldnames(Options);

% This is how many variable arguments you put in
nargs=length(varargin);

% This guarantees that for a given parameter, there is a parameter value.
if round(nargs/2)~=nargs/2
    error('Error: You must enter optional arguments as parameter name/parameter value pairs.')
end

% This goes through the variable arguments and assigns their new values
% according to the input.
for Pair=reshape(varargin,2,[])
    InputName=(Pair{1}); 
    if any(strncmp(InputName,(OptionNames),100)) % This sees if any of the inputs match the possible options
        
        if strcmp(InputName,'PlotElement')
            if ischar(Pair{2})~=1
                error('PlotElement option needs values of either ''On'' or ''Off''')
            end
        end
            
        Options.(InputName)=Pair{2}; % This assigns the value 
    else
        error([InputName,' is not a valid option'])
    end
end


% Parameters
% N=200; % Number of points to calculate
% lambda=800e-9;
% L=4; % Total length propagated
% n=1.5; % Index of refraction
z=zi:L/N:L+zi; % Which z values to plot
zf=z(end);

% Initial q
% q1=1i*(pi*n*(300e-6)^2)/lambda;

% Initialize variables
qf=zeros(N+1,1);
w=zeros(N+1,2);
R=zeros(N+1,2);

% Matrix
j=1;
for x=z-zi;
M=[1, x/n ;
   0,  1 ];

% Propagation Equation
qf(j,1)=(M(1,1)*qi+M(1,2))...
       /(M(2,1)*qi+M(2,2));

% For each calculated q, find the waist and radius of curvature
w(j,1)=sqrt(-lambda/(pi*n*imag(1/qf(j,1))));
w(j,2)=x;
R(j,1)=1/real(1/qf(j,1));
R(j,2)=x;
j=j+1;
end

% This loop finds any waists and labels them on the plot
for j=2:length(w)-1
    if w(j,1) < w(j-1,1)
        if w(j,1) < w(j+1,1)
            Waist = w(j,1);
            z_Waist = z(j);
            WaistFound=1;
            break
        else 
            WaistFound=0;
        end
    else
        WaistFound=0;
    end
end
    
if strcmp(Options.PlotElement,'On')
    figure(Options.PlotHandle)
    % These plot the beam profile
    plot(z,w(:,1),Options.PlotColor)
    hold all
    plot(z,-w(:,1),Options.PlotColor)
    hold all
    % This line plots the position of the beam waist if there is one
    if WaistFound==1
        line([z_Waist,z_Waist],[Waist, -Waist],'LineStyle','--','Color',Options.PlotColor)
    end
end

end















