% Gaussian Beam Propagation - Lens
%
% This program propagates a gaussian beam through a lens.  It can be used
% in combination with the other 'GBeam' propagating programs to find the q
% parameter along a distance with several optical elements.  The position
% of the lens is plotted and the focal length is labeled.
%
% [qf]=GBeam_Lens(qi,lambda,n,f,phi,PlotHandle,z,TanOrSag)
%
% REQUIRED INPUTS
%
% qi = Inital q parameter
%
% lambda = Wavelength of beam in vacuum
% 
% n = Index of refraction of medium the lens is in.  This is just used to
%     find a value of the waist at the position of the lens so something
%     can be plotted.
%
% f = Focal length of lens
%
% phi = Angle (in radians) lens is tilted in plane of incidence
%
%
% z = Position of the lens.  Used to plot the position of the lens when
%     propagating a beam through several elements.
%
% TanOrSag = This input needs to either be 'Tan' or 'Sag' to indicate
%            whether the tangential or sagittal plane is being calculated.
%
% VARIABLE INPUTS
%
% PlotElement = This needs to be followed by 'yes' or 'no'.  It determines
%                whether or not to plot the element on the plot specified
%                by 'PlotHandle'.
%
% PlotHandle = This is used to identify the plot that is being used so that
%              several elements can be plotted together.
%
% OUTPUTS
%
% qf = The final q parameter after the lens
%
%

function [qf]=GBeam_Lens(qi,lambda,n,f,phi,z,TanOrSag,varargin)

% This are the possible options available with a default value
Options=struct('PlotElement','on',...
               'PlotHandle',1);

% This stores just the option names to use later
OptionNames=fieldnames(Options);

% This is how many variable arguments you put in
nargs=length(varargin);

% This guarantees that for a given parameter, there is a parameter value.
if round(nargs/2)~=nargs/2
    error('Error: You must enter optional arguments as parameter name/parameter value pairs.')
end

% This goes through the variable arguments and assigns their new values
% according to the input.
for Pair=reshape(varargin,2,[])
    InputName=(Pair{1}); 
    if any(strncmp(InputName,(OptionNames),100)) % This sees if any of the inputs match the possible options
        
        if strcmp(InputName,'PlotElement')
            if ischar(Pair{2})~=1
                error('PlotElement option needs values of either ''On'' or ''Off''')
            end
        end
            
        Options.(InputName)=Pair{2}; % This assigns the value 
    else
        error([InputName,' is not a valid option'])
    end
end

if strcmp(TanOrSag,'Tan')
    M=[       1,        0  ;
       -1/(f*cos(phi)), 1 ];
elseif strcmp(TanOrSag,'Sag')
    M=[     1,       0;
       -cos(phi)/f, 1];
end

% Propagation Equation
qf=(M(1,1)*qi+M(1,2))...
 ./(M(2,1)*qi+M(2,2));

% Find a waist so the lens can be plotted in comparison to something
w=sqrt(-lambda./(pi*n*imag(1./qf)));

if strcmp(Options.PlotElement,'On')
    figure(Options.PlotHandle)
    line([z,z],[1.1*w,-1.1*w],'Color','b')
    hold all
    text(z,1.2*w,['f=',num2str(f*1e3),' mm'],'HorizontalAlignment','center','VerticalAlignment','top','Interpreter','Latex')
end

end















